import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import pystan as pyst
import seaborn as sns
import patsy
sns.set(context='talk', style='ticks', font_scale=1.2, rc={'figure.figsize': (6.5, 5.5), 'xtick.direction': 'in', 'ytick.direction': 'in'})

# read in data
cumulativeData = pd.read_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Data/cumulativeData.csv')
cumulativeData = cumulativeData[cumulativeData['Notes'].isnull()]
# remove gap 5
cumulativeData = cumulativeData[cumulativeData['gap']!=5]
# calculate yearly growth
cumulativeData['Year1_growth'] = cumulativeData.apply(lambda x: 100*(x['end_2013']-x['initial_2013'])/x['initial_2013'] if x['gap']==8 else 100*(x['end_2012']-x['initial_2012'])/x['initial_2012'], axis=1)
cumulativeData['Year2_growth'] = cumulativeData.apply(lambda x: 100*(x['end_2014']-x['initial_2014'])/x['initial_2014'] if x['gap']==8 else 100*(x['end_2013']-x['initial_2013'])/x['initial_2013'], axis=1)
cumulativeData['Year3_growth'] = cumulativeData.apply(lambda x: np.nan if x['gap']==8 else 100*(x['end_2014']-x['initial_2014'])/x['initial_2014'], axis=1)

germlings2012 = cumulativeData[cumulativeData['initial_2012'] < 15]
germlings2013 = cumulativeData[(cumulativeData['initial_2013'] < 15) & (cumulativeData['initial_2012'].isnull())]
germlings2014 = cumulativeData[(cumulativeData['initial_2014'] < 15) & (cumulativeData['initial_2013'].isnull()) & (cumulativeData['initial_2012'].isnull())]
germlings = pd.concat([germlings2012, germlings2013, germlings2014], axis=0)

# extract the relevant data
paredData = germlings[['gap', 'plot', 'treatment', 'tagID', 'scientific', 'Year1_growth', 'Year2_growth', 'Year3_growth']]
# put into long format
paredLong = pd.melt(paredData, id_vars=['gap', 'plot', 'treatment', 'tagID', 'scientific'], var_name='GapYear', value_name='Growth')
common = ['Acer rubrum', 'Fagus grandifolia',  'Fraxinus', 'Liriodendron tulipifera', 'Liquidambar styraciflua']
paredLong = paredLong[paredLong['scientific'].isin(common)]
# drop na
paredLong.dropna(inplace=True)

# check the data replication
print paredLong.groupby(['scientific', 'GapYear', 'treatment'])['Growth'].count()

paredLong['spPlot'] = pd.factorize(paredLong['scientific'] + paredLong['plot'] + paredLong['GapYear'])[0] + 1
paredLong.sort(columns=['spPlot'], inplace=True)
paredLong.groupby(['scientific', 'GapYear', 'treatment', 'spPlot'])['Growth'].count().to_csv('/Users/Nate/Desktop/test.csv')

growthModel = """
data{
	int<lower=0> N;		// number of observations
	int<lower=0> K;
	int<lower=0> J;		// number of species
	vector[N] y;			// response variable
	int plotSp[N];
	int species[N];			// species identifier
	row_vector[6] X[K];		// design matrix
	int sp_plot[K];
	matrix[6,6] R;			// priors for covariance matrix
}
parameters{
	vector<lower=0>[J] sd_species;		// each species get's a unique sd
	vector[K] plotSp_mu;
	real<lower=0> sd_plot;
	vector[6] B[J];					// a unique vector matrix for each species
	vector[6] B_mu;				// overall coefficients
	cov_matrix[6] Tau;				// covariance matrix
}
transformed parameters{
	vector[K] plotSp_hat;
	vector<lower=0>[N] sd_y;

	for(n in 1:N){
		sd_y[n] <- sd_species[species[n]];
	}

	for(k in 1:K){
		plotSp_hat[k] <- X[k]*B[sp_plot[k]];			// plot/species/year means are species-specific functions of pesticide and gap age
	}
}
model{
	for(n in 1:N){
		y[n] ~ normal(plotSp_mu[plotSp[n]], sd_y[n]);		// germlings vary around their plot/species/year specific means
	}
	
	plotSp_mu ~ normal(plotSp_hat, sd_plot);			// plot/species/year means vary around their predicted values (see above)

	for(j in 1:J){
		B[j] ~ multi_normal(B_mu, Tau);	// species-level coefficients vary normally around overall coefs
	}

	// PRIORS
	B_mu ~ normal(0, 1);	// Priors on overall effects
	Tau ~ inv_wishart(7, R);	// Priors on covariance of effects
}
generated quantities{	
}
"""
growthMod_comp = pyst.StanModel(model_code=growthModel)

# data setup
spMap = {'Acer rubrum': 1, 'Fagus grandifolia': 2,  'Fraxinus': 3, 'Liriodendron tulipifera': 4, 'Liquidambar styraciflua': 5}

plotData = paredLong.drop_duplicates('spPlot')
X = patsy.dmatrix('~C(treatment)*C(GapYear)', plotData)
X_names = X.design_info.column_names

growData = {
	'N': len(paredLong),
	'K': len(plotData),
	'J': len(paredLong['scientific'].unique()),
	'y': (paredLong['Growth'] - paredLong['Growth'].mean())/paredLong['Growth'].std(),
	'plotSp': paredLong['spPlot'],
	'species': paredLong['scientific'].map(spMap),
	'X': np.array(X),
	'sp_plot': plotData['scientific'].map(spMap),
	'R': np.eye(6)
}

growFitted = growthMod_comp.sampling(data=growData, iter=10000, chains=4, thin=10)

s = pd.DataFrame(growFitted.extract('plotSp_mu')['plotSp_mu'])
s.plot(kind='kde', legend=False)
plt.show()

#### SPECIES-LEVEL TRENDS ####
B = growFitted.extract('B')['B']

Maple_coefs = pd.DataFrame(B[:,0,:])
Maple_coefs.columns = X_names
Maple_coefs.plot(kind='kde', legend=False)
plt.show()
Maple_coefs['Species'] = 'Acer rubrum'

Beech_coefs = pd.DataFrame(B[:,1,:])
Beech_coefs.columns = X_names
Beech_coefs.plot(kind='kde', legend=False)
plt.show()
Beech_coefs['Species'] = 'Fagus grandifolia'

Ash_coefs = pd.DataFrame(B[:,2,:])
Ash_coefs.columns = X_names
Ash_coefs.plot(kind='kde', legend=False)
plt.show()
Ash_coefs['Species'] = 'Fraxinus'

Tulip_coefs = pd.DataFrame(B[:,3,:])
Tulip_coefs.columns = X_names
Tulip_coefs.plot(kind='kde', legend=False)
plt.show()
Tulip_coefs['Species'] = 'Liriodendron tulipifera'

Sweet_coefs = pd.DataFrame(B[:,4,:])
Sweet_coefs.columns = X_names
Sweet_coefs.plot(kind='kde', legend=False)
plt.show()
Sweet_coefs['Species'] = 'Liquidambar styraciflua'

species_coefs = pd.concat([Maple_coefs, Beech_coefs, Ash_coefs, Tulip_coefs, Sweet_coefs])
species_coefs.to_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Growth/species_coefs.csv', index=False)
species_coefs = pd.read_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Growth/species_coefs.csv')

f,ax = plt.subplots(1,5, figsize=(15, 5.5))
parmNames = ['Intercept', 'Pesticide', 'Year 2', 'Year 3', 'Pesticide:Year 2', 'Pesticide:Year 3']
for i in range(5):
	currentSp = common[i]
	tempData = species_coefs[species_coefs['Species']==currentSp]
	ax[i].axvline(0, c='k', ls='--')
	ax[i].hlines(range(6)[::-1], tempData.quantile(0.025), tempData.quantile(0.975), lw=1)
	ax[i].hlines(range(6)[::-1], tempData.quantile(0.1), tempData.quantile(0.9), lw=2)
	[ax[i].text(x,y,s, ha='left', va='center') for x,y,s in zip(tempData.quantile(0.98), range(6)[::-1], np.round(np.mean(tempData<0), 3).astype('str'))]
	ax[i].plot(tempData.median(), range(6)[::-1], 'o', ms=10, mew=1, mfc='w')
	ax[i].set_ylim([-0.5, 5.5])
	ax[i].set_yticks(range(6))
	ax[i].set_title(currentSp.replace(' ', '\n'))
	ax[i].set_xlim([-1.5, 1.5])
	ax[i].set_xticks([-1.5, -0.5, 0.5, 1.5])
	if i==0:
		ax[i].set_yticklabels(parmNames[::-1])
	else:
		ax[i].set_yticklabels('')

sns.despine()
plt.subplots_adjust(wspace=0.3)
plt.savefig('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Growth/growth_coefs.pdf')
plt.show()

# species plots
cMap = {'Control': ['white', 'Herbivores'], 'Pesticide': ['black', 'No Herbivores']}

plotMeans = pd.DataFrame(growFitted.extract('plotSp_mu')['plotSp_mu']).median()*paredLong['Growth'].std() + paredLong['Growth'].mean()
plotData.reset_index(inplace=True)
plotData['scientific'].replace('Fraxinus', 'Fraxinus spp.', inplace=True)
plotData['EstMeans'] = plotMeans
plotSumm = plotData.groupby(['scientific', 'GapYear', 'treatment'])['EstMeans'].agg({'mean': lambda x: x.mean(), 'SE': lambda x: x.std()/np.sqrt(x.count())})
common = ['Acer rubrum', 'Fagus grandifolia',  'Fraxinus spp.', 'Liriodendron tulipifera', 'Liquidambar styraciflua']

f = plt.figure(figsize=(10, 10))
for i in range(5):
	spData = plotSumm.loc[common[i]]
	spData['location'] = [0.9, 1.1, 1.9, 2.1, 2.9, 3.1]
	ax = f.add_subplot(2, 3, i+1)
	for treat in ['Control', 'Pesticide']:
		trtData = spData.xs(treat, level=1)
		ax.errorbar(trtData['location'], np.array(trtData['mean']), yerr=trtData['SE'], fmt='o', 
			mew=1, mfc=cMap[treat][0], label=cMap[treat][1], ms=15, capsize=0, ecolor='k')
	ax.set_ylim([-5, 100])
	ax.set_xlim([0.5, 3.5])
	ax.set_xticks([1, 2, 3])
	ax.set_title(common[i].replace(' ', '\n'), style='italic')
	if i==0:
		ax.legend(fontsize=12, markerscale=0.8, loc=2)
	if i in [2, 3,4]:
		ax.set_xticklabels(['Year 1', 'Year 2', 'Year 3'])
	else:
		ax.set_xticklabels('')
	if i not in [1,2,4]:
		ax.set_ylabel('Relative Growth Rate\n(% change in height per year)')

plt.subplots_adjust(wspace=0.3, hspace=0.3)
sns.despine()
plt.savefig('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Growth/species_growth.pdf')
plt.show()

# acer rubrum
Maple_coefs = species_coefs[species_coefs['Species']=='Acer rubrum']
#pesticide effect in year 1
pestEff_y1 = Maple_coefs.iloc[:,1]*paredLong['Growth'].std()
print np.round(pestEff_y1.quantile([0.025, 0.5, 0.975]), 3)
# pesticide effect in year 2
pest_Y2 = (Maple_coefs.iloc[:,1] + Maple_coefs.iloc[:,4])*paredLong['Growth'].std()
print np.mean(pest_Y2 > 0)
# pesticide effect in year 2
pest_Y3 = (Maple_coefs.iloc[:,1] + Maple_coefs.iloc[:,5])*paredLong['Growth'].std()
print np.mean(pest_Y3 > 0)

# fagus grandifolia
# pesticide effect in year 2
pest_Y2 = (Beech_coefs.iloc[:,1] + Beech_coefs.iloc[:,4])*paredLong['Growth'].std()
print np.mean(pest_Y2 < 0)

# fraxinus
pest_Y2 = (Ash_coefs.iloc[:,1] + Ash_coefs.iloc[:,4])*paredLong['Growth'].std()
print np.mean(pest_Y2 < 0)

# liriodendron tulipifera
pest_Y2 = (Tulip_coefs.iloc[:,1] + Tulip_coefs.iloc[:,4])*paredLong['Growth'].std()
print np.mean(pest_Y2 < 0)

# sweet gum
pest_Y1 = (Sweet_coefs.iloc[:,1])*paredLong['Growth'].std()
print np.mean(pest_Y1 > 0)
print np.round(pest_Y1.quantile([0.025, 0.5, 0.975]), 3)

pest_Y2 = (Sweet_coefs.iloc[:,1] + Sweet_coefs.iloc[:,4])*paredLong['Growth'].std()
print np.mean(pest_Y2 > 0)

pest_Y3 = (Sweet_coefs.iloc[:,1] + Sweet_coefs.iloc[:,5])*paredLong['Growth'].std()
print np.mean(pest_Y3 > 0)
print np.round(pest_Y3.quantile([0.025, 0.5, 0.975]), 3)

#### OVERALL COEFS
B_mu = pd.DataFrame(growFitted.extract('B_mu')['B_mu'])
B_mu.columns = X_names
B_mu.plot(kind='kde', legend=False)
plt.show()
B_mu.to_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Growth/overall_coefs.csv', index=False)
B_mu = pd.read_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Growth/overall_coefs.csv')

print B_mu.iloc[:,1].quantile([0.025, 0.5, 0.975])
print np.mean(B_mu.iloc[:,1] > 0)

print B_mu.iloc[:,2].quantile([0.025, 0.5, 0.975])
print np.mean(B_mu.iloc[:,2] < 0)

print B_mu.iloc[:,3].quantile([0.025, 0.5, 0.975])
print np.mean(B_mu.iloc[:,3] < 0)