import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as st
import pystan as pyst
import seaborn as sns
import patsy
sns.set(context='talk', style='ticks', font_scale=1.2, rc={'figure.figsize': (6.5, 5.5), 'xtick.direction': 'in', 'ytick.direction': 'in'})

# read in data
cumulativeData = pd.read_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Data/cumulativeData.csv')
cumulativeData = cumulativeData[cumulativeData['Notes'].isnull()]
# remove gap 5
cumulativeData = cumulativeData[cumulativeData['gap']!=5]
# totalRecruits
recruitSumms = cumulativeData.groupby(['scientific', 'treatment', 'gap', 'plot'], as_index=False)['total_recruit'].sum()
# isolate common species
common = ['Acer rubrum', 'Fagus grandifolia',  'Fraxinus', 'Liriodendron tulipifera', 'Liquidambar styraciflua']
recruitSumms = recruitSumms[recruitSumms['scientific'].isin(common)]

def diffFunc(x):
	P1 =  x[x['treatment']=='Pesticide']['total_recruit'].apply(float)
	P2 = x[x['treatment']=='Control']['total_recruit'].apply(float)
	if len(x)==1:
		return pd.DataFrame({'S': x['scientific'].unique(), 'P': x['gap'].unique(), 'Diff': np.nan})
	else:
		return pd.DataFrame({'S': x['scientific'].unique(), 'P': x['gap'].unique(), 'Diff': P1.values-P2.values})

t = recruitSumms.groupby(['scientific', 'gap'], as_index=False).apply(diffFunc)

recruitModel = """
data{
	int<lower=0> N; 		// number of observations
	int<lower=0> J;		// number of species
	int y[N];			// response variable
	int species[N];			// species identifier
	vector[N] Herb;
}
parameters{
	vector[J] B0;
	vector[J] B1;
	real B0_mu;
	real B1_mu;
	real<lower=0> sd_b0;
	real<lower=1> sd_b1;
}
transformed parameters{
	vector[N] theta;
	vector[N] yhat;

	for(n in 1:N){
		theta[n] <- B0[species[n]] + B1[species[n]]*Herb[n];
		yhat[n] <- exp(theta[n]);
	}
}
model{
	y ~ poisson(yhat);

	B0 ~ normal(B0_mu, sd_b0);
	B1 ~ normal(B1_mu, sd_b1);
}
generated quantities{	
}
"""
recMod_comp = pyst.StanModel(model_code=recruitModel)

# data setup
spMap = {'Acer rubrum': 1, 'Fagus grandifolia': 2,  'Fraxinus': 3, 'Liriodendron tulipifera': 4, 'Liquidambar styraciflua': 5}
herbMap = {'Control': 0, 'Pesticide': 1}

recrData = {
	'N': len(recruitSumms),
	'J': len(recruitSumms['scientific'].unique()),
	'Herb': recruitSumms['treatment'].map(herbMap),
	'y': recruitSumms['total_recruit'],
	'species': recruitSumms['scientific'].map(spMap)
}

recFitted = recMod_comp.sampling(data=recrData, iter=10000, chains=4, thin=10)

#### SPECIES-LEVEL TRENDS ####
B1 = recFitted.extract('B1')['B1']
B0 = recFitted.extract('B0')['B0']


Maple_coefs = pd.DataFrame({'Int': B0[:,0], 'Eff': B1[:,0]})
Maple_coefs.plot(kind='kde', legend=False)
plt.show()
Maple_coefs['Species'] = 'Acer rubrum'

Beech_coefs = pd.DataFrame({'Int': B0[:,1], 'Eff': B1[:,1]})
Beech_coefs.plot(kind='kde', legend=True)
plt.show()
Beech_coefs['Species'] = 'Fagus grandifolia'

Ash_coefs = pd.DataFrame({'Int': B0[:,2], 'Eff': B1[:,2]})
Ash_coefs.plot(kind='kde', legend=False)
plt.show()
Ash_coefs['Species'] = 'Fraxinus'

Tulip_coefs = pd.DataFrame({'Int': B0[:,3], 'Eff': B1[:,3]})
Tulip_coefs.plot(kind='kde', legend=False)
plt.show()
Tulip_coefs['Species'] = 'Liriodendron tulipifera'

Sweet_coefs = pd.DataFrame({'Int': B0[:,4], 'Eff': B1[:,4]})
Sweet_coefs.plot(kind='kde', legend=False)
plt.show()
Sweet_coefs['Species'] = 'Liquidambar styraciflua'

species_coefs = pd.concat([Maple_coefs, Beech_coefs, Ash_coefs, Tulip_coefs, Sweet_coefs])
species_coefs.to_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Recruitment/species_coefs.csv', index=False)
species_coefs = pd.read_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Recruitment/species_coefs.csv')

f,ax = plt.subplots(1,5, figsize=(15, 5.5))
parmNames = ['Intercept', 'Pesticide']
for i in range(5):
	currentSp = common[i]
	tempData = species_coefs[species_coefs['Species']==currentSp]
	ax[i].axvline(0, c='k', ls='--')
	ax[i].hlines(range(2), tempData.quantile(0.025), tempData.quantile(0.975), lw=1)
	ax[i].hlines(range(2), tempData.quantile(0.1), tempData.quantile(0.9), lw=2)
	[ax[i].text(x,y,s, ha='left', va='center') for x,y,s in zip(tempData.quantile(0.98), range(2), np.round(np.mean(tempData<0), 3).astype('str'))]
	ax[i].plot(tempData.median(), range(2), 'o', ms=10, mew=1, mfc='w')
	ax[i].set_ylim([-0.5, 1.5])
	ax[i].set_yticks(range(2))
	ax[i].set_title(currentSp.replace(' ', '\n'))
	ax[i].set_xlim([-2.5, 2.5])
	if i==0:
		ax[i].set_yticklabels(parmNames[::-1])
	else:
		ax[i].set_yticklabels('')
sns.despine()
plt.subplots_adjust(wspace=0.3)
plt.savefig('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Recruitment/recruit_coefs.pdf')
plt.show()

# species plots
recruitSumms['scientific'].replace('Fraxinus', 'Fraxinus spp.', inplace=True)
plotSumm = recruitSumms.groupby(['scientific', 'treatment'])['total_recruit'].agg({'mean': lambda x: x.mean(), 'SE': lambda x: np.sqrt(x.mean())/np.sqrt(x.count())})
common = ['Acer rubrum', 'Fagus grandifolia',  'Fraxinus spp.', 'Liriodendron tulipifera', 'Liquidambar styraciflua']

f = plt.figure(figsize=(10, 10))
for i in range(5):
	spData = plotSumm.loc[common[i]]
	spData['location'] = [0, 1]
	ax = f.add_subplot(2, 3, i+1)
	ax.errorbar(spData['location'], spData['mean'], yerr=spData['SE'], fmt='o', 
		mew=1, ms=0, mfc='white', capsize=0, ecolor='k')
	ax.scatter(spData['location'], spData['mean'], s=300, c=['w', 'k'], zorder=100, linewidths=1)
	ax.set_ylim([0, 4])
	ax.set_xlim([-0.5, 1.5])
	ax.set_xticks([0, 1])
	ax.set_title(common[i].replace(' ', '\n'), style='italic')
	if i==0:
		ax.legend(fontsize=12, markerscale=0.8, loc=2)
	if i in [2, 3,4]:
		ax.set_xticklabels(['Herbivores', 'No\nHerbivores'])
	else:
		ax.set_xticklabels('')
	if i not in [1,2,4]:
		ax.set_ylabel('Number of Recruits')
	if i==3:
		ax.set_ylim([0, 16])
plt.subplots_adjust(wspace=0.3, hspace=0.3)
sns.despine()
plt.savefig('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Recruitment/recruitment.pdf')
plt.show()

# acer rubrum
Maple_coefs = species_coefs[species_coefs['Species']=='Acer rubrum']
#pesticide effect
print np.round(Maple_coefs.iloc[:,1].quantile([0.025, 0.5, 0.975]), 3)
# diff from year 1 to year 2
print Maple_coefs.iloc[:,2].quantile([0.025, 0.5, 0.975])
# diff from year 1 to year 3
print Maple_coefs.iloc[:,3].quantile([0.025, 0.5, 0.975])
# diff from year 2 to year 3 
print 1-np.mean((Maple_coefs.iloc[:,2] - Maple_coefs.iloc[:,3]) < 0)
print (Maple_coefs.iloc[:,2] - Maple_coefs.iloc[:,3]).quantile([0.025, 0.5, 0.975])

# liriodendron tulipifera
Tulip_coefs = species_coefs[species_coefs['Species']=='Liriodendron tulipifera']
#pesticide effect
print np.round(Tulip_coefs.iloc[:,1].quantile([0.025, 0.5, 0.975]), 3)
# diff from year 1 to year 2
print Tulip_coefs.iloc[:,2].quantile([0.025, 0.5, 0.975])
# diff from year 1 to year 3
print Tulip_coefs.iloc[:,3].quantile([0.025, 0.5, 0.975])
# diff from year 2 to year 3 
print 1-np.mean((Tulip_coefs.iloc[:,2] - Tulip_coefs.iloc[:,3]) < 0)
print (Tulip_coefs.iloc[:,2] - Tulip_coefs.iloc[:,3]).quantile([0.025, 0.5, 0.975])


#### OVERALL COEFS
B_mu = pd.DataFrame(mortFitted.extract('B_mu')['B_mu'])
B_mu.columns = X_names
B_mu.plot(kind='kde', legend=False)
plt.show()
B_mu.to_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Mortality/overall_coefs.csv', index=False)
B_mu = pd.read_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Mortality/overall_coefs.csv')

print B_mu.iloc[:,1].quantile([0.025, 0.5, 0.975])
print np.mean(B_mu.iloc[:,1] < 0)

print B_mu.iloc[:,2].quantile([0.025, 0.5, 0.975])
print np.mean(B_mu.iloc[:,2] < 0)

print B_mu.iloc[:,3].quantile([0.025, 0.5, 0.975])
print np.mean(B_mu.iloc[:,3] < 0)